@@ -606,7 +606,7 @@ pdht_get_aloc(const shared_file_t *sf, const kuid_t *key)
 		char buf[sizeof(uint64)];
 		int len;
 
-		len = ggept_filesize_encode(shared_file_size(sf), buf);
+		len = ggept_filesize_encode(shared_file_size(sf), buf, sizeof buf);
 		g_assert(len > 0 && UNSIGNED(len) <= sizeof buf);
 		ok = ok && ggep_stream_pack(&gs, GGEP_NAME(length), buf, len, 0);
 	}
@@ -618,7 +618,7 @@ pdht_get_aloc(const shared_file_t *sf, const kuid_t *key)
 			char buf[sizeof(uint64)];
 			int len;
 
-			len = ggept_filesize_encode(fi->done, buf);
+			len = ggept_filesize_encode(fi->done, buf, sizeof buf);
 			g_assert(len > 0 && UNSIGNED(len) <= sizeof buf);
 			ok = ok && ggep_stream_pack(&gs, GGEP_NAME(avail), buf, len, 0);
 		}
@@ -1281,7 +1281,7 @@ pdht_prox_done(void *u_arg, pdht_error_t code, const pdht_info_t *info)
 			time_delta_t elapsed =
 				delta_time(tm_time(), pdht_proxy.last_publish);
 			if (elapsed > DHT_VALUE_PROX_EXPIRE)
-				gnet_stats_count_general(GNR_DHT_REPUBLISHED_LATE, +1);
+				gnet_stats_inc_general(GNR_DHT_REPUBLISHED_LATE);
 		}
 
 		delay = publisher_delay(info, DHT_VALUE_PROX_EXPIRE);
