@@ -62,131 +62,6 @@
  * There can be up to 2^5 = 32 keys per version.
  */
 
-static const char *keys_096[] = {
-	"261c 78d6 fcc5 d96e 2649 061a 4534 29b5",
-	"2629 7de4 8edd 43eb 6c47 2b01 caf1 5e86",
-	"50c2 076a 5a15 5c0c 27fb eda0 381b 2eb7",
-	"851c 2fff 0a31 c6ad 2181 4d31 8fea 492c",
-	"c8f8 01a8 2975 cc75 417c 63aa 5403 5b41",
-	"045b aca8 5227 7d0f 232a 7c6a d713 d5dd",
-	"f281 f0c5 23fb cf66 5ca4 6a3d 9df1 dc6a",
-	"0fc8 ac1f 76da 5f7e 3459 bd7d 3175 76cf",
-	"f981 7fe7 06d1 d3d9 9d69 1e47 b8d0 9adf",
-	"7422 4730 d7d0 9293 002c b700 8979 dccf",
-	"c328 4be8 9008 8d52 cbd6 2f45 30ba 9467",
-	"cdc2 2db6 6bba 312c 10fb 246b b371 be09",
-	"017a 3e68 90e0 e0f0 8124 3cc8 fcf8 3bf7",
-	"2e56 a817 02b3 0819 d971 a245 c33e 42fc",
-	"0ee7 8801 db48 f2d6 64ad 6c42 bac3 f7ee",
-	"c758 af82 e6a3 aa5f 1da0 c127 4541 1ce8",
-	"2edc 2b16 9e66 a191 9e45 2e66 ea98 0c7b",
-	"438a a8ed d27e 711e 631e 2372 a013 d095",
-	"45cf 2974 2086 d00e efec 9277 05a3 bff2",
-	"bb86 594c 74e2 432d 5444 8a85 82c8 d098",
-	"64f4 9829 a541 8625 578c fd90 639c f42b",
-	"3084 a2bc f4ed 8b3c 2a2b 1834 cd8e 3f8b",
-};
-
-static const char *keys_096_1[] = {
-	"ac3e f7b1 af37 e22c ce69 f25f 8dd2 8e51",
-	"7c66 fddd f8d9 8bde 8c6c 072a 1935 2237",
-	"6aab e420 921d b32c 09e9 34b8 e403 525d",
-	"3014 53b9 64fd 95a5 e52b e9c9 99f6 323e",
-	"dacd 5a3d 34e5 d280 fd58 1af0 6fef ac72",
-	"020a 4163 41f6 f089 b285 0321 48df 44fb",
-	"8059 f2a1 e91a f319 2ad2 e13d 6634 6eb6",
-	"4853 1dba a1bf 9386 8b24 af94 f112 9e5b",
-	"e053 3cd9 87ba 9dd7 c4c4 c32e 9bd2 a61a",
-	"8625 b5d3 b531 18b9 8716 403d fa26 4a5b",
-	"433c 60fb eb1b c33b 139b 1594 9c69 91e4",
-	"4a47 e00b 2933 f634 d194 6376 b777 fbb8",
-	"70ec 73f6 7ee3 ac83 a899 5e84 1d82 371c",
-	"2b53 cad1 b7d6 ad9e f4a1 96bd 8c37 264e",
-	"20ed fcbc feb8 f96d e037 af6f 2486 ff84",
-	"109c 5d02 d4df 33ac ead3 fe7c 8bc2 e87f",
-	"1392 547e 3abe b83c 3e15 8416 0b7b 5d89",
-	"a546 f15e 797e 8081 9d88 0c82 5afc 4c2e",
-	"fe97 9602 dcd7 efb6 74ee 35ba ea4a 9158",
-	"396c 5dcc 43a3 9577 a610 9e89 7079 5862",
-	"9e54 e398 f5b0 1e02 272d 87a7 e36e 10a2",
-	"9536 535d 0e1b e017 2f73 a0bc c01a dcf9",
-};
-
-static const char *keys_096_2[] = {
-	"fa04 15ab 5f1d 52cc 5a5d 865b 27b5 c5a5",
-	"7f0b 5987 86ec a100 85c5 d0fc f504 b58c",
-	"2b06 f3a7 05a8 2769 2679 1ab9 9ba0 2360",
-	"4c3f 5636 3ba7 471b 43ea 3350 d71f b0fd",
-	"00bf c06b 3703 78e3 87c2 ea7a 3a8c b684",
-	"5103 6580 7be1 9cab fecb 3c46 12a9 4287",
-	"a5c5 a8f0 d10e 2202 526a f62e d615 afe2",
-	"32b8 5d94 5e4b c936 8703 2386 5f67 aa6b",
-	"10f8 e454 5d88 ec00 7e44 636f cda5 e45a",
-	"0409 a3aa b426 c79a 0d17 8895 f595 d744",
-	"9f9f c223 b290 0e7b 0950 d991 1ed9 bae0",
-	"a297 f754 a185 81f1 d57d 7b82 fde8 c7d1",
-	"d3de 3537 3271 0649 c2e8 de86 842e 1b79",
-	"86d1 3b44 b4eb 4842 1736 d42b 8b0b ce30",
-	"3b93 8172 1331 99fe 1423 c188 642b c45b",
-	"f013 188b 833d de3b 49d9 3709 6067 fd88",
-	"42ad 9a84 808e 7638 601a 6d33 9543 0827",
-	"ba4e d65e 1768 320d c88f 7b16 3194 0e08",
-	"ef2b e292 e29f c2ef 96a2 bc3c fbcb bb96",
-	"71a7 a828 ce50 d736 11a5 e687 ad42 ce5c",
-	"266b be05 0d52 f825 37e8 ffe3 b4f1 282c",
-	"e75d b41f 2332 309d c3ee 4dd7 060d fc08",
-};
-
-static const char *keys_096_3[] = {
-	"ca43 91c3 d080 c596 5ec2 9e17 5011 ebb6",
-	"4b53 6e49 a6a8 c045 c0c4 bfc7 8c54 4d9a",
-	"411b fc41 6643 ba3a 527a 52c9 4a72 3db0",
-	"13aa 0ae0 54f8 c9b3 2af8 3faa c363 cc4b",
-	"ebd2 69db d313 b7da 2425 9412 1ce3 16be",
-	"faf3 1626 20db 8b5a d448 0887 a22e c156",
-	"e925 4acd 65d3 f64d daea 0766 b260 ae03",
-	"62ff b5f7 02bf 189c a17f 7fec 3845 2d8e",
-	"efe6 4c84 2149 aee7 0e61 9192 b293 8f5d",
-	"47ec defc f33c 698e a8d4 eb06 c5bd d062",
-	"4aab bc51 4caf f6c2 676d f60c 6e6e 6ad6",
-	"afad 3bbc 4981 0a4d a0b9 a247 697f 8052",
-	"c244 2aff d148 3a7b 3da0 f837 7207 6795",
-	"8b8b 3562 29c5 3d4c 3e9a 46ca 47ff 5e18",
-	"4973 9c6e bbb0 6a26 29e9 92d6 300b c4cb",
-	"c9fd b78f 37d5 029d 7e99 ce84 d052 7efb",
-	"1520 f5bd 1a53 8250 ca3e 8ced 4d1f 1b38",
-	"850a 2b7b f978 16fa 3465 a243 c66f 3796",
-	"fdf1 fbdf 54bb 32a0 44b1 3892 30b4 ea5c",
-	"8a44 58ba ddac b04f 9806 1bce 7a7d 486e",
-	"6cc1 768e 0d38 e314 c388 88be 7bf9 2dea",
-	"c994 a380 dde0 baff 8900 c9e5 d94c b9ff",
-};
-
-static const char *keys_096_4[] = {
-	"4313 0c48 3747 a2b5 580c f7a5 7adc d83f",
-	"94c7 eeca e5c1 c4c4 a09d 243c b846 6917",
-	"3188 446a 39da fca8 11ae ef30 8073 f61a",
-	"146a 985b 20bc 639e 971e 5fd4 304f f02b",
-	"a99f a005 5c50 bf6a 721f 7c88 e4d4 3a1c",
-	"e28c 2482 40b0 a41d 4d30 1326 1227 20b1",
-	"070a 52ce af8f 2d26 5d01 c5bc 356e 9e3c",
-	"f018 d34c 91a6 4653 d397 7bcc 3d64 7b15",
-	"bc64 25b4 ac36 0fce 7097 9809 31da 71b6",
-	"4e40 d31e d36d 1ade 53ec 0a37 21a0 10fd",
-	"1dbb 4288 a191 8016 1bf8 956f 36aa c9c8",
-	"45be 9349 07c7 2455 ed07 ab54 f014 fd32",
-	"047c 4878 4809 00a2 bc3c 48bf 701c 2123",
-	"225d da36 c6ef 96db deb4 ad03 dba6 ce97",
-	"c4e9 6cbf 2f1c fd6f 1047 57fc 4c8b 8058",
-	"d131 9d1e 161a bbdb d39c 2e0a 1b5e 5546",
-	"a0e1 36b2 52d1 efec 6649 608a a3ba 687a",
-	"46b7 0f59 b70a 8ce4 147f 18c2 4307 6a30",
-	"b8e7 c642 de0c 9498 bdf0 5afe ae6c 8d21",
-	"6352 4957 2f40 56f2 0bfe 44c6 173a 9615",
-	"4ec5 9106 ab7c 019b 4b87 348a 13eb 82f2",
-	"a2d0 b172 becb 8516 8f17 b2f4 80e2 4488",
-};
-
 static const char *keys_096_5[] = {
 	"29af ab9b fc6b 58b7 9031 cf87 f7cc 3c63",
 	"5dee 3d3f efb1 ac8d 27f4 2fa9 b1c3 705b",
@@ -495,6 +370,37 @@ static const char *keys_098_3[] = {
 	"0cb7 2795 f88d 77d3 0dd5 d978 a818 1253",
 };
 
+static const char *keys_098_4[] = {
+	"1a94 ebfa bce6 1878 3dfe d484 62ab 20ef",
+	"f1f6 8df3 fa99 8805 c248 c008 e793 11eb",
+	"c785 e618 7a5f e3d1 1a02 483a d334 e706",
+	"8213 787a 5ce4 a016 84aa 8147 c3c0 10a8",
+	"0916 b1d0 6e33 6746 dda7 4980 50f3 4786",
+	"0699 29ec 7855 0952 0301 4676 4db3 0efe",
+	"0c9f 34f2 bb70 73b7 6115 777e 3339 ede0",
+	"fa09 d4b4 19d5 f151 6057 8da5 a18f c38a",
+	"4e7e 84c1 4b72 cdbf 1b20 4267 a877 4262",
+	"ea85 070f d6c6 4c52 aaad f375 6b68 59f7",
+	"06c5 ac96 f7c9 2de3 3c16 9b6e 5e04 2c82",
+	"f8d6 981f 83b8 9fde fc4d 1d5b 2e52 2186",
+	"9b38 2bec 6740 009a 8a4b 143f d786 c6f9",
+	"3617 bff5 b5b6 86e6 bf0c 0560 3372 577b",
+	"088f fd18 1a6f b22f 9db3 3419 196d 4544",
+	"f3aa 2b69 0567 6193 0c14 a331 82dc be14",
+	"0d66 0ab5 bd7d 9174 ede8 2b53 31f5 74b8",
+	"a66f 698a 6514 760f 6e37 6de3 237a 40a1",
+	"24de 4a61 90f4 eba3 ba42 7a09 1fa4 69db",
+	"3996 3984 cca8 cfab 28df 5af0 5576 3bf7",
+	"8a8a a3fe 0950 3346 848d 388a aaf5 5b0a",
+	"82ca de6a 9485 2655 3b0a c67a f02e 0543",
+	"8ddf 1a8c 5b96 d7e0 e43e 63cb 04c3 dc8d",
+	"d4bf dd41 c177 6ae1 fc7f dbf4 4d16 d93d",
+	"34e8 5a53 e51e b7dd e3d8 8965 6888 6998",
+	"4566 2c9b c0ff 06ee 2d65 8037 8ad6 d0d2",
+	"eb20 c9dc 68fa 1176 a138 20d6 befa fec5",
+	"aeed 4f98 40ca 8dfa 3488 d8c4 6b70 92c7",
+};
+
 /**
  * Describes the keys to use depending on the version.
  */
@@ -504,26 +410,6 @@ struct tokkey {
 	uint count;			/**< Amount of keys defined */
 } token_keys[] = {
 	/* Keep this array sorted by increasing timestamp */
-	{
-		{ 0, 96, 0, '\0', 0, 0, 1138057200 },		/* 2006-01-24 */
-		keys_096, G_N_ELEMENTS(keys_096),
-	},
-	{
-		{ 0, 96, 1, '\0', 0, 0, 1140562800 },		/* 2006-02-22 */
-		keys_096_1, G_N_ELEMENTS(keys_096_1),
-	},
-	{
-		{ 0, 96, 2, '\0', 0, 0, 1161025877 },		/* 2006-10-16 */
-		keys_096_2, G_N_ELEMENTS(keys_096_2),
-	},
-	{
-		{ 0, 96, 3, '\0', 0, 0, 1163108781 },		/* 2006-11-09 */
-		keys_096_3, G_N_ELEMENTS(keys_096_3),
-	},
-	{
-		{ 0, 96, 4, '\0', 0, 0, 1183775000 },		/* 2007-07-07 */
-		keys_096_4, G_N_ELEMENTS(keys_096_4),
-	},
 	{
 		{ 0, 96, 5, '\0', 0, 0, 1207083000 },		/* 2008-04-01 */
 		keys_096_5, G_N_ELEMENTS(keys_096_5),
@@ -569,6 +455,10 @@ struct tokkey {
 		{ 0, 98, 3, '\0', 0, 0, 1338760800 },		/* 2012-06-04 */
 		keys_098_3, G_N_ELEMENTS(keys_098_3),
 	},
+	{
+		{ 0, 98, 4, '\0', 0, 0, 1352588400 },		/* 2012-11-11 */
+		keys_098_4, G_N_ELEMENTS(keys_098_4),
+	},
 };
 
 /**
@@ -706,9 +596,8 @@ find_tokkey_version(const version_t *ver, time_t now)
 	 * If we're more recent, we probably have a stripped list of past key
 	 * sets, and therefore cannot validate their token.
 	 *
-	 * All versions after we switched to git are can be checked provided
-	 * we still have a copy of the keys that they had at the time they
-	 * were released.
+	 * All versions after we switched to git can be checked provided we still
+	 * have a copy of the keys that they had at the time they were released.
 	 */
 
 	if (
