@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2011 Raphael Manfredi <Raphael_Manfredi@pobox.com>
+ * Copyright (c) 2009-2012 Raphael Manfredi <Raphael_Manfredi@pobox.com>
  * All rights reserved.
  *
  * Copyright (c) 2006 Christian Biere <christianbiere@gmx.de>
@@ -37,32 +37,40 @@
  *
  * A simple hashtable implementation.
  *
- * There are two interesting properties in this hash table:
+ * There are three interesting properties in this hash table:
  *
  * - The items and the internal data structures are allocated out of a
  *   same contiguous memory region (aka the "arena").
  *
  * - Memory for the arena is allocated directly through the VMM layer.
  *
+ * - The access interface can be dynamically configured to be thread-safe.
+ *
  * As such, this hash table is suitable for being used by low-level memory
  * allocators.
  *
  * @author Raphael Manfredi
- * @date 2009-2011
+ * @date 2009-2012
  * @author Christian Biere
  * @date 2006
  */
 
 #include "common.h"
 
-#include "lib/hashtable.h"
-#include "lib/atomic.h"
-#include "lib/mutex.h"
-#include "lib/vmm.h"
-#include "lib/xmalloc.h"
+#include "hashtable.h"
+#include "atomic.h"
+#include "entropy.h"
+#include "hashing.h"
+#include "mutex.h"
+#include "omalloc.h"
+#include "pow2.h"
+#include "spinlock.h"
+#include "vmm.h"
+#include "xmalloc.h"
 
-#include "lib/override.h"		/* Must be the last header included */
+#include "override.h"		/* Must be the last header included */
 
+#define HASH_ITEMS_BINS			2	/* Initial amount of bins */
 #define HASH_ITEMS_PER_BIN		4
 #define HASH_ITEMS_GROW			56
 
@@ -94,14 +102,20 @@ struct hash_table {
 	mutex_t external_lock;		/* Lock for external atomic operations */
 	size_t num_items;			/* Array length of "items" */
 	size_t num_bins;			/* Number of bins */
+	size_t bin_bits;			/* Number of bits to fold hashed value to */
 	size_t num_held;			/* Number of items actually in the table */
 	size_t bin_fill;			/* Number of bins in use */
-	hash_table_hash_func hash;	/* Key hash functions, or NULL */
-	hash_table_eq_func eq;		/* Key equality function, or NULL */
+	hash_fn_t hash;				/* Key hash functions, or NULL */
+	eq_fn_t eq;					/* Key equality function, or NULL */
 	hash_item_t **bins;			/* Array of bins of size ``num_bins'' */
 	hash_item_t *free_list;		/* List of free hash items */
 	hash_item_t *items;			/* Array of items */
-#ifdef TRACK_VMM
+	/*
+	 * Lookup caching.
+	 */
+	const void *last_key;		/* Last looked-up key */
+	hash_item_t *last_item;		/* Last looked-up item (NULL if invalid) */
+	size_t last_bin;			/* Last bin where looked-up key belongs */
 	/*
 	 * Since we use these data structures during tracking, be careful:
 	 * if the table is created with the _real variant, it is used by
@@ -109,60 +123,97 @@ struct hash_table {
 	 * layer specially.
 	 */
 	unsigned real:1;			/* If TRUE, created as "real" */
-#endif
+	unsigned not_leaking:1;		/* Don't track allocated VMM regions */
 	unsigned special:1;			/* Set if structure allocated specially */
 	unsigned readonly:1;		/* Set if data structures protected */
 	unsigned thread_safe:1;		/* Set if table must be thread-safe */
+	unsigned once:1;			/* Object allocated using "once" memory */
+	unsigned self_keys:1;		/* Keys are self-representing */
 };
 
+/**
+ * Avoid complexity attacks on the hash table.
+ *
+ * A random number is used to perturb the hash value for all the keys so
+ * that no attack on the hash table insertion complexity can be made, such
+ * as presenting a set of keys that will pathologically make insertions
+ * O(n) instead of O(1) on average.
+ */
+static unsigned hash_offset;
+
+/**
+ * Minimal amount of bins we we want (power of two) that can fill up one page.
+ */
+static size_t hash_min_bins;
+
+/**
+ * Initialize hash offset if not already done.
+ */
+static G_GNUC_COLD void
+hash_offset_init(void)
+{
+	static spinlock_t offset_slk = SPINLOCK_INIT;
+	static bool done;
+
+	if G_UNLIKELY(!done) {
+		spinlock(&offset_slk);
+		if (!done) {
+			/* Don't allocate any memory, hence can't call arc4random() */
+			hash_offset = entropy_random();
+			done = TRUE;
+			/* Memory barrier will be done by spinunlock() */
+		}
+		spinunlock(&offset_slk);
+	}
+}
+
 static inline void *
 hash_vmm_alloc(const struct hash_table *ht, size_t size)
 {
 #ifdef TRACK_VMM
-	if (ht->real)
+	if (ht->real || ht->not_leaking)
 		return vmm_alloc_notrack(size);
 	else
-		return vmm_alloc(size);
 #else
-	(void) ht;
-	return vmm_alloc(size);
-#endif
+	{
+		(void) ht;
+		return vmm_alloc(size);
+	}
+#endif	/* TRACK_VMM */
 }
 
 static inline void
 hash_vmm_free(const struct hash_table *ht, void *p, size_t size)
 {
 #ifdef TRACK_VMM
-	if (ht->real)
+	if (ht->real || ht->not_leaking)
 		vmm_free_notrack(p, size);
 	else
+#else
+	{
+		(void) ht;
 		vmm_free(p, size);
-#else
-	(void) ht;
-	vmm_free(p, size);
-#endif
+	}
+#endif	/* TRACK_VMM */
 }
 
 static inline void
-hash_mark_real(hash_table_t * ht, gboolean is_real)
+hash_mark_real(hash_table_t *ht, bool is_real)
 {
-#ifdef TRACK_VMM
 	ht->real = booleanize(is_real);
-#else
-	(void) ht;
-	(void) is_real;
-#endif
 }
 
 static inline void
-hash_copy_real_flag(hash_table_t *dest, const hash_table_t *src)
+hash_mark_once(hash_table_t *ht, bool is_once)
+{
+	ht->once = booleanize(is_once);
+}
+
+static inline void
+hash_copy_flags(hash_table_t *dest, const hash_table_t *src)
 {
-#ifdef TRACK_VMM
 	dest->real = src->real;
-#else
-	(void) dest;
-	(void) src;
-#endif
+	dest->not_leaking = src->not_leaking;
 }
 
 static inline void
@@ -173,18 +224,18 @@ hash_table_check(const struct hash_table *ht)
 	g_assert(ht->num_bins > 0 && ht->num_bins < SIZE_MAX / 2);
 }
 
-/**
- * NOTE: A naive direct use of the pointer has a much worse distribution e.g.,
- *		 only a quarter of the bins are used.
- */
-static inline size_t
+static inline unsigned
 hash_id_key(const void *key)
 {
-	size_t n = (size_t) key;
-	return ((0x4F1BBCDCUL * (guint64) n) >> 32) ^ n;
+	/*
+	 * A naive direct use of the pointer has a much worse distribution,
+	 * e.g. only a quarter of the bins are used.
+	 */
+
+	return GOLDEN_RATIO_32 * pointer_to_ulong(key);
 }
 
-static inline gboolean
+static inline bool
 hash_id_eq(const void *a, const void *b)
 {
 	return a == b;
@@ -243,7 +294,7 @@ hash_bins_items_arena_size(const hash_table_t *ht, size_t *items_offset)
 
 static void
 hash_table_new_intern(hash_table_t *ht,
-	size_t num_bins, hash_table_hash_func hash, hash_table_eq_func eq)
+	size_t num_bins, hash_fn_t hash, eq_fn_t eq)
 {
 	size_t i;
 	size_t arena;
@@ -252,14 +303,39 @@ hash_table_new_intern(hash_table_t *ht,
 	g_assert(ht);
 	g_assert(num_bins > 1);
 
+	hash_offset_init();
+
 	ht->magic = HASHTABLE_MAGIC;
 	ht->num_held = 0;
 	ht->bin_fill = 0;
-	ht->hash = hash ? hash : hash_id_key;
-	ht->eq = eq ? eq : hash_id_eq;
+	ht->hash = hash != NULL ? hash : hash_id_key;
+	ht->eq = eq != NULL ? eq : hash_id_eq;
+	ht->self_keys = booleanize(hash_id_eq == ht->eq);
 
-	ht->num_bins = num_bins;
+	/*
+	 * Since the arena is going to be held in a VMM page with nothing
+	 * else, make sure we're filling the page as much as we can.
+	 */
+
+	if G_UNLIKELY(0 == hash_min_bins) {
+		size_t n;
+
+		/* No spinlock, at worst we'll do this computation more than once */
+
+		n = compat_pagesize() / (sizeof ht->bins[0] +
+			HASH_ITEMS_PER_BIN * sizeof ht->items[0]);
+		hash_min_bins = 1 << highest_bit_set(n);
+
+		g_assert(hash_min_bins > 1);
+		g_assert(IS_POWER_OF_2(hash_min_bins));
+	}
+
+	ht->num_bins = MAX(num_bins, hash_min_bins);
 	ht->num_items = ht->num_bins * HASH_ITEMS_PER_BIN;
+	ht->bin_bits = highest_bit_set64(ht->num_bins);
+
+	g_assert(IS_POWER_OF_2(ht->num_bins));
+	g_assert((1UL << ht->bin_bits) == ht->num_bins);
 
 	arena = hash_bins_items_arena_size(ht, &items_off);
 
@@ -288,14 +364,13 @@ hash_table_new_intern(hash_table_t *ht,
 }
 
 hash_table_t *
-hash_table_new_full(hash_table_hash_func hash, hash_table_eq_func eq)
+hash_table_new_full(hash_fn_t hash, eq_fn_t eq)
 {
-	hash_table_t *ht = xpmalloc(sizeof *ht);
+	hash_table_t *ht = xpmalloc0(sizeof *ht);
 
 	g_assert(ht);
 
-	ZERO(ht);
-	hash_table_new_intern(ht, 2, hash, eq);
+	hash_table_new_intern(ht, HASH_ITEMS_BINS, hash, eq);
 	return ht;
 }
 
@@ -329,24 +404,24 @@ static inline ALWAYS_INLINE void
 ht_synchronize(const hash_table_t *ht)
 {
 	if (ht->thread_safe) {
-		hash_table_t *wht = deconstify_gpointer(ht);
-		mutex_get(&wht->lock);
+		hash_table_t *wht = deconstify_pointer(ht);
+		mutex_lock(&wht->lock);
 		g_assert(HASHTABLE_MAGIC == ht->magic);
 	}
 }
 
 #define ht_return(ht,v) G_STMT_START {					\
 	if (ht->thread_safe) {								\
-		hash_table_t *wht = deconstify_gpointer(ht);	\
-		mutex_release(&wht->lock);						\
+		hash_table_t *wht = deconstify_pointer(ht);		\
+		mutex_unlock(&wht->lock);						\
 	}													\
 	return v;											\
 } G_STMT_END
 
 #define ht_return_void(ht) G_STMT_START {				\
 	if (ht->thread_safe) {								\
-		hash_table_t *wht = deconstify_gpointer(ht);	\
-		mutex_release(&wht->lock);						\
+		hash_table_t *wht = deconstify_pointer(ht);		\
+		mutex_unlock(&wht->lock);						\
 	}													\
 	return;												\
 } G_STMT_END
@@ -364,20 +439,16 @@ hash_table_arena_memory(const hash_table_t *ht)
 	ht_return(ht, ret);
 }
 
-/**
- * NOTE: A naive direct use of the pointer has a much worse distribution e.g.,
- *		only a quarter of the bins are used.
- */
-static inline size_t
+static inline unsigned
 hash_key(const hash_table_t *ht, const void *key)
 {
-	return (*ht->hash)(key);
+	return (*ht->hash)(key) + hash_offset;
 }
 
-static inline gboolean
+static inline bool
 hash_eq(const hash_table_t *ht, const void *a, const void *b)
 {
-	return (*ht->eq)(a, b);
+	return a == b || (*ht->eq)(a, b);
 }
 
 /**
@@ -393,19 +464,41 @@ static hash_item_t *
 hash_table_find(const hash_table_t *ht, const void *key, size_t *bin)
 {
 	hash_item_t *item;
-	size_t hash;
+	size_t idx;
 
 	hash_table_check(ht);
 
-	hash = hash_key(ht, key) & (ht->num_bins - 1);
-	item = ht->bins[hash];
+	/*
+	 * Caching of last successful lookup result for self-representing keys.
+	 */
+
+	if (ht->last_item != NULL && ht->self_keys && ht->last_key == key) {
+		if (bin != NULL)
+			*bin = ht->last_bin;
+		return ht->last_item;
+	}
+
+	/*
+	 * Lookup key in the hash table.
+	 */
+
+	idx = hashing_fold(hash_key(ht, key), ht->bin_bits);
+	item = ht->bins[idx];
 	if (bin) {
-		*bin = hash;
+		*bin = idx;
 	}
 
 	for ( /* NOTHING */ ; item != NULL; item = item->next) {
-		if (hash_eq(ht, key, item->key))
+		if G_LIKELY(hash_eq(ht, key, item->key)) {
+			if (ht->self_keys) {
+				/* Cache successful lookup result */
+				hash_table_t *wht = deconstify_pointer(ht);
+				wht->last_key = key;
+				wht->last_bin = idx;
+				wht->last_item = item;
+			}
 			return item;
+		}
 	}
 
 	return NULL;
@@ -414,20 +507,17 @@ hash_table_find(const hash_table_t *ht, const void *key, size_t *bin)
 /**
  * Iterate over the hashtable, invoking the "func" callback on each item
  * with the additional "data" argument.
- *
- * @attention
- * This call is not thread-safe atomically and requires explicit locking
- * to perform mutual exclusion.
  */
 void
-hash_table_foreach(const hash_table_t *ht,
-	hash_table_foreach_func func, void *data)
+hash_table_foreach(const hash_table_t *ht, ckeyval_fn_t func, void *data)
 {
 	size_t i, n;
 
 	hash_table_check(ht);
 	g_assert(func != NULL);
 
+	ht_synchronize(ht);
+
 	n = ht->num_held;
 	i = ht->num_bins;
 
@@ -435,11 +525,13 @@ hash_table_foreach(const hash_table_t *ht,
 		hash_item_t *item;
 
 		for (item = ht->bins[i]; NULL != item; item = item->next) {
-			(*func)(item->key, deconstify_gpointer(item->value), data);
+			(*func)(item->key, deconstify_pointer(item->value), data);
 			n--;
 		}
 	}
 	g_assert(0 == n);
+
+	ht_return_void(ht);
 }
 
 /**
@@ -474,6 +566,7 @@ hash_table_clear(hash_table_t *ht)
 
 	ht->num_held = 0;
 	ht->bin_fill = 0;
+	ht->last_item = NULL;	/* Clear lookup cache */
 
 	ht_return_void(ht);
 }
@@ -498,6 +591,8 @@ hash_table_reset(hash_table_t *ht)
 	ht->num_held = 0;
 	ht->num_items = 0;
 	ht->free_list = NULL;
+	ht->last_key = NULL;
+	ht->last_item = NULL;
 }
 
 /**
@@ -506,7 +601,7 @@ hash_table_reset(hash_table_t *ht)
  *
  * @return FALSE if the item could not be added, TRUE on success.
  */
-static gboolean
+static bool
 hash_table_insert_no_resize(hash_table_t *ht,
 	const void *key, const void *value)
 {
@@ -541,7 +636,7 @@ hash_table_insert_no_resize(hash_table_t *ht,
 static void
 hash_table_resize_helper(const void *key, void *value, void *data)
 {
-	gboolean ok;
+	bool ok;
 	ok = hash_table_insert_no_resize(data, key, value);
 	g_assert(ok);
 }
@@ -552,7 +647,7 @@ hash_table_resize(hash_table_t *ht, size_t n)
 	hash_table_t tmp;
 
 	ZERO(&tmp);
-	hash_copy_real_flag(&tmp, ht);
+	hash_copy_flags(&tmp, ht);
 	hash_table_new_intern(&tmp, n, ht->hash, ht->eq);
 	hash_table_foreach(ht, hash_table_resize_helper, &tmp);
 
@@ -566,6 +661,7 @@ hash_table_resize(hash_table_t *ht, size_t n)
 	ht->num_items = tmp.num_items;
 	ht->num_held = tmp.num_held;
 	ht->bin_fill = tmp.bin_fill;
+	ht->bin_bits = tmp.bin_bits;
 	ht->free_list = tmp.free_list;
 }
 
@@ -602,10 +698,10 @@ hash_table_resize_on_insert(hash_table_t *ht)
  *
  * @return FALSE if the item could not be added, TRUE on success.
  */
-gboolean
+bool
 hash_table_insert(hash_table_t *ht, const void *key, const void *value)
 {
-	gboolean ret;
+	bool ret;
 
 	hash_table_check(ht);
 	ht_synchronize(ht);
@@ -637,7 +733,7 @@ hash_table_status(const hash_table_t *ht)
  *
  * @return TRUE if item was present in the hash table.
  */
-gboolean
+bool
 hash_table_remove(hash_table_t *ht, const void *key)
 {
 	hash_item_t *item;
@@ -674,6 +770,9 @@ hash_table_remove(hash_table_t *ht, const void *key)
 		hash_item_free(ht, item);
 		ht->num_held--;
 
+		if G_UNLIKELY(item == ht->last_item)
+			ht->last_item = NULL;	/* Clear lookup cache */
+
 		safety_assert(!hash_table_lookup(ht, key));
 
 		hash_table_resize_on_remove(ht);
@@ -708,28 +807,37 @@ hash_table_replace(hash_table_t *ht, const void *key, const void *value)
 }
 
 /**
+ * Lookup key in the table.
+ *
  * @return value associated with the key.
  */
 void *
 hash_table_lookup(const hash_table_t *ht, const void *key)
 {
 	hash_item_t *item;
+	void *p;
 
 	hash_table_check(ht);
 	ht_synchronize(ht);
 
 	item = hash_table_find(ht, key, NULL);
+	p = item ? deconstify_pointer(item->value) : NULL;
 
-	ht_return(ht, item ? deconstify_gpointer(item->value) : NULL);
+	ht_return(ht, p);
 }
 
 /**
  * Lookup key in the hash table, returning physical pointers to the key/value
  * items into ``kp'' and ``vp'' respectively, if non-NULL.
  *
+ * @param ht		the hash table
+ * @param key		the key to lookup
+ * @param kp		where the key pointer is copied, if non-NULL
+ * @param vp		where the value pointer is copied, if non-NULL
+ *
  * @return TRUE if item was found.
  */
-gboolean
+bool
 hash_table_lookup_extended(const hash_table_t *ht,
 	const void *key, const void **kp, void **vp)
 {
@@ -746,19 +854,20 @@ hash_table_lookup_extended(const hash_table_t *ht,
 	if (kp)
 		*kp = item->key;
 	if (vp)
-		*vp = deconstify_gpointer(item->value);
+		*vp = deconstify_pointer(item->value);
 
 	ht_return(ht, TRUE);
 }
 
 /**
  * Check whether hashlist contains the key.
+ *
  * @return TRUE if the key is present.
  */
-gboolean
+bool
 hash_table_contains(const hash_table_t *ht, const void *key)
 {
-	gboolean ret;
+	bool ret;
 
 	hash_table_check(ht);
 	ht_synchronize(ht);
@@ -767,6 +876,65 @@ hash_table_contains(const hash_table_t *ht, const void *key)
 	ht_return(ht, ret);
 }
 
+/**
+ * Iterate over the hashtable, invoking the "func" callback on each item
+ * with the additional "data" argument and removing the item if the
+ * callback returns TRUE.
+ *
+ * @return the amount of items removed from the table.
+ */
+size_t
+hash_table_foreach_remove(hash_table_t *ht, ckeyval_rm_fn_t func, void *data)
+{
+	size_t i, n, old_n, removed = 0;
+
+	hash_table_check(ht);
+	g_assert(func != NULL);
+
+	ht_synchronize(ht);
+
+	n = old_n = ht->num_held;
+	i = ht->num_bins;
+
+	while (i-- > 0) {
+		hash_item_t *item, *next = NULL, *prev = NULL;
+
+		for (item = ht->bins[i]; NULL != item; item = next) {
+			next = item->next;
+			if ((*func)(item->key, deconstify_pointer(item->value), data)) {
+				/* Remove the item from the table */
+				if (item == ht->bins[i]) {
+					if (NULL == next) {
+						g_assert(ht->bin_fill > 0);
+						ht->bin_fill--;
+					}
+					ht->bins[i] = next;
+				} else {
+					g_assert(prev != NULL);
+					g_assert(prev->next == item);
+
+					prev->next = next;
+				}
+				hash_item_free(ht, item);
+				ht->num_held--;
+				if G_UNLIKELY(item == ht->last_item)
+					ht->last_item = NULL;	/* Clear lookup cache */
+				removed++;
+			} else {
+				prev = item;	/* Item was kept, becoming new previous item */
+			}
+			n--;
+		}
+	}
+	g_assert(0 == n);
+	g_assert(old_n == removed + ht->num_held);
+
+	if (removed != 0)
+		hash_table_resize_on_remove(ht);
+
+	ht_return(ht, removed);
+}
+
 /**
  * Destroy hash table, reclaiming all the space.
  */
@@ -776,6 +944,7 @@ hash_table_destroy(hash_table_t *ht)
 	hash_table_check(ht);
 	ht_synchronize(ht);
 	g_assert(!ht->special);
+	g_assert(!ht->once);
 
 	hash_table_reset(ht);
 	if (ht->thread_safe) {
@@ -874,7 +1043,7 @@ hash_table_lock(hash_table_t *ht)
 	hash_table_check(ht);
 	g_assert(ht->thread_safe);
 
-	mutex_get(&ht->external_lock);
+	mutex_lock(&ht->external_lock);
 }
 
 /**
@@ -889,7 +1058,7 @@ hash_table_unlock(hash_table_t *ht)
 	hash_table_check(ht);
 	g_assert(ht->thread_safe);
 
-	mutex_release(&ht->external_lock);
+	mutex_unlock(&ht->external_lock);
 }
 
 /**
@@ -919,7 +1088,7 @@ struct ht_linearize {
 	void **array;
 	size_t count;
 	size_t i;
-	gboolean keys;
+	bool keys;
 };
 
 /**
@@ -932,14 +1101,14 @@ hash_table_linearize_item(const void *key, void *value, void *data)
 
 	g_assert(htl->i < htl->count);
 
-	htl->array[htl->i++] = htl->keys ? deconstify_gpointer(key) : value;
+	htl->array[htl->i++] = htl->keys ? deconstify_pointer(key) : value;
 }
 
 /**
  * Linearize the keys/values into dynamically allocated array.
  */
 static void **
-hash_table_linearize(const hash_table_t *ht, size_t *count, gboolean keys)
+hash_table_linearize(const hash_table_t *ht, size_t *count, bool keys)
 {
 	struct ht_linearize htl;
 
@@ -1007,13 +1176,56 @@ hash_table_values(const hash_table_t *ht, size_t *count)
 	return hash_table_linearize(ht, count, FALSE);
 }
 
+/**
+ * Compute the clustering factor of the hash table.
+ *
+ * If there are ``n'' items spread overe ``m'' bins, each bin should have
+ * n/m items.
+ *
+ * We can measure the clustering factor ``c'' by computing for each bin i the
+ * value Bi = size(bin #i)^2 / n.  Then c = (Sum Bi) - n/m + 1.
+ *
+ * If each bin has the theoretical value, Bi = (n/m)^2 / n = n/m^2.
+ * Hence c = m * n/m^2 - n/m + 1 = 1.
+ *
+ * If c > 1, then it means there is clustering occurring, the higher the value
+ * the higher the clustering.  If c < 1, then the hash function disperses
+ * values more efficiently than a pure random function!
+ */
+double
+hash_table_clustering(const hash_table_t *ht)
+{
+	size_t i, n, m;
+	double c = 0.0;
+
+	hash_table_check(ht);
+
+	ht_synchronize(ht);
+
+	n = ht->num_held;
+	m = i = ht->num_bins;
+
+	while (i-- > 0) {
+		hash_item_t *item;
+		size_t j = 0;
+
+		for (item = ht->bins[i]; NULL != item; item = item->next)
+			j++;
+
+		if (j != 0)
+			c += j*j / (double) n;
+	}
+
+	ht_return(ht, 1.0 + c - n / (0 == m ? 1.0 : (double) m));
+}
+
 /**
  * Create "special" hash table, where the object is allocated through the
  * specified allocator.
  */
 hash_table_t *
 hash_table_new_special_full(const hash_table_alloc_t alloc, void *obj,
-	hash_table_hash_func hash, hash_table_eq_func eq)
+	hash_fn_t hash, eq_fn_t eq)
 {
 	hash_table_t *ht = (*alloc)(obj, sizeof *ht);
 
@@ -1021,7 +1233,7 @@ hash_table_new_special_full(const hash_table_alloc_t alloc, void *obj,
 
 	ZERO(ht);
 	ht->special = booleanize(TRUE);
-	hash_table_new_intern(ht, 2, hash, eq);
+	hash_table_new_intern(ht, HASH_ITEMS_BINS, hash, eq);
 	return ht;
 }
 
@@ -1031,6 +1243,23 @@ hash_table_new_special(const hash_table_alloc_t alloc, void *obj)
 	return hash_table_new_special_full(alloc, obj, NULL, NULL);
 }
 
+hash_table_t *
+hash_table_new_full_not_leaking(hash_fn_t hash, eq_fn_t eq)
+{
+	hash_table_t *ht = omalloc0(sizeof *ht);
+
+	ht->not_leaking = booleanize(TRUE);
+	ht->once = booleanize(TRUE);
+	hash_table_new_intern(ht, HASH_ITEMS_BINS, hash, eq);
+	return ht;
+}
+
+hash_table_t *
+hash_table_new_not_leaking(void)
+{
+	return hash_table_new_full_not_leaking(NULL, NULL);
+}
+
 /*
  * The hash table is used to keep track of the malloc() and free() operations,
  * so we need special routines to ensure allocation and freeing of memory
@@ -1044,19 +1273,43 @@ hash_table_new_special(const hash_table_alloc_t alloc, void *obj)
 #undef malloc
 #undef free
 
-hash_table_t *
-hash_table_new_full_real(hash_table_hash_func hash, hash_table_eq_func eq)
+static hash_table_t *
+hash_table_new_full_real_using(hash_table_t *ht, bool once,
+	hash_fn_t hash, eq_fn_t eq)
 {
-	hash_table_t *ht = malloc(sizeof *ht);
-
-	g_assert(ht);
+	g_assert(ht != NULL);
 
 	ZERO(ht);
 	hash_mark_real(ht, TRUE);
-	hash_table_new_intern(ht, 2, hash, eq);
+	hash_mark_once(ht, once);
+	hash_table_new_intern(ht, HASH_ITEMS_BINS, hash, eq);
 	return ht;
 }
 
+hash_table_t *
+hash_table_once_new_full_real(hash_fn_t hash, eq_fn_t eq)
+{
+	hash_table_t *ht = omalloc(sizeof *ht);
+
+	return hash_table_new_full_real_using(ht, TRUE, hash, eq);
+}
+
+hash_table_t *
+hash_table_new_full_real(hash_fn_t hash, eq_fn_t eq)
+{
+	hash_table_t *ht = malloc(sizeof *ht);
+
+	return hash_table_new_full_real_using(ht, FALSE, hash, eq);
+}
+
+hash_table_t *
+hash_table_once_new_real(void)
+{
+	hash_table_t *ht = omalloc(sizeof *ht);
+
+	return hash_table_new_full_real_using(ht, TRUE, NULL, NULL);
+}
+
 hash_table_t *
 hash_table_new_real(void)
 {
@@ -1068,6 +1321,7 @@ hash_table_destroy_real(hash_table_t *ht)
 {
 	hash_table_check(ht);
 	ht_synchronize(ht);
+	g_assert(!ht->once);
 
 	hash_table_reset(ht);
 	if (ht->thread_safe) {
