@@ -26,6 +26,8 @@
 
 #include "common.h"
 
+#include "xmalloc.h"
+
 /**
  * Allocates an array of "struct iov" elements.
  * @param n The desired array length in elements.
@@ -39,7 +41,7 @@ iov_alloc_n(size_t n)
 		g_assert_not_reached(); /* We don't want to handle failed allocations */
 		return NULL;
 	}
-	iov = g_malloc(n * sizeof *iov);
+	iov = xmalloc0(n * sizeof *iov);
 	return iov;
 }
 
@@ -55,6 +57,15 @@ iov_get(void *base, size_t size)
 	return iov;
 }
 
+/**
+ * Free array of "struct iov" elements allocated via iov_alloc_n().
+ */
+static inline void
+iov_free(iovec_t *iov)
+{
+	xfree(iov);
+}
+
 /**
  * Resets an array of "struct iov" elements, so that iov_base is NULL
  * and iov_len is 0 for each element.
