@@ -197,57 +197,6 @@ rand31_current_seed(void)
 	return rand31_seed;
 }
 
-/**
- * Compute uniformly distributed random number in the [0, max] range,
- * avoiding any modulo bias, using the specified random function to generate
- * the numbers.
- *
- * @param rf	function generating 31-bit wide numbers
- * @param max	maximum allowed value for the result (inclusive)
- *
- * @return uniformly distributed 31-bit number from 0 to max, inclusive.
- */
-int
-rand31_upto(rand31_fn_t rf, unsigned max)
-{
-	unsigned range, min;
-	int i;
-
-	g_assert(rf != NULL);
-	g_assert(max <= INT_MAX);
-
-	if G_UNLIKELY(0 == max)
-		return 0;
-
-	if G_UNLIKELY(INT_MAX == max)
-		return (*rf)();
-
-	/*
-	 * See random_upto() for details on modulo bias and how our
-	 * strategy restores a uniform distribution.
-	 *
-	 * The code here is simpler because there cannot be any overflow on
-	 * the 32-bit unsigned value.
-	 */
-
-	range = max + 1;
-
-	if (is_pow2(range))
-		return (*rf)() & max;		/* max = range - 1 */
-
-	min = (1U << 31) % range;
-
-	for (i = 0; i < 100; i++) {
-		unsigned value = (*rf)();
-
-		if (value >= min)
-			return value % range;
-	}
-
-	s_error("no luck with random number generator %s()",
-		stacktrace_routine_name(func_to_pointer(rf), FALSE));
-}
-
 /**
  * Compute uniformly distributed random number in the [0, max] range,
  * avoiding any modulo bias.
@@ -257,7 +206,11 @@ rand31_upto(rand31_fn_t rf, unsigned max)
 int
 rand31_value(unsigned max)
 {
-	return rand31_upto(rand31, max);
+	g_assert(0 == (max & (1U << 31)));	/* bit 31 is zero */
+
+	STATIC_ASSERT(sizeof(int) == sizeof(unsigned));	/* Hence safe cast below */
+
+	return random_upto((random_fn_t) rand31, max);
 }
 
 /**
