@@ -676,6 +676,15 @@ symbols_check_consistency(symbols_t *st)
 	htable_t *sym_pc;
 	const void *main_pc;
 
+	/*
+	 * Reset the values we're computing since we can be called multiple
+	 * times when we try to load symbols from multiple sources.
+	 */
+
+	st->garbage = FALSE;
+	st->offset = 0;
+	st->mismatch = FALSE;
+
 	if (0 == st->count)
 		return;
 
@@ -778,8 +787,8 @@ symbols_check_consistency(symbols_t *st)
 	 * find the proper symbols.
 	 */
 
-	if (offset != 0 && mismatches != 0)
-		s_warning("BUG in %s()", G_STRFUNC);
+	g_soft_assert_log(0 == offset || 0 == mismatches,
+		"offset=%zu, mismatches=%zu", offset, mismatches);
 
 done:
 	htable_free_null(&sym_pc);
@@ -832,7 +841,7 @@ symbols_parse_nm(symbols_t *st, char *line)
  * @param exe	the executable path, to assess freshness of nm file
  * @param nm	the path to the nm file, symbols from the executable
  *
- * @return opened file if successfull, NULL on error with the error already
+ * @return opened file if successful, NULL on error with the error already
  * logged appropriately.
  */
 static FILE *
@@ -944,11 +953,26 @@ symbols_load_from(symbols_t *st, const char *exe, const  char *lpath)
 
 	if (!has_bfd) {
 		size_t rw;
+		const char meta[] = "$&`;()<>|";
+		const char *p = exe;
+		int c;
+
+		/*
+		 * Make sure there are no problematic shell meta-characters in the path.
+		 */
+
+		while ((c = *p++)) {
+			if (strchr(meta, c)) {
+				s_warning("found shell meta-character '%c' in path \"%s\", "
+					"not loading symbols", c, exe);
+				goto use_pre_computed;
+			}
+		}
 
 		rw = str_bprintf(tmp, sizeof tmp, "nm -p %s", exe);
 		if (rw != strlen(exe) + CONST_STRLEN("nm -p ")) {
 			s_warning("full path \"%s\" too long, cannot load symbols", exe);
-			goto done;
+			goto use_pre_computed;
 		}
 
 		f = popen(tmp, "r");
@@ -980,14 +1004,14 @@ retry:
 	 * Try to open the symbols from the installed nm file.
 	 */
 
-#ifndef MINGW32
 use_pre_computed:
-#endif
 
-	if (!retried && 0 == st->count) {
+	if (!retried && (0 == st->count || st->garbage)) {
 		char *nm = make_pathname(ARCHLIB_EXP, NM_FILE);
 
-		s_warning("no symbols loaded, trying with pre-computed \"%s\"", nm);
+		s_warning("%s, trying with pre-computed \"%s\"",
+			0 == st->count ? "no symbols loaded" : "garbage symbols", nm);
+
 		st->fresh = FALSE;
 		f = symbols_open(st, exe, nm);
 		retried = TRUE;
@@ -999,6 +1023,9 @@ use_pre_computed:
 			goto retry;
 		}
 
+		if (st->garbage)
+			return;			/* Already went through the "done" part */
+
 		/* FALL THROUGH */
 	}
 
@@ -1013,6 +1040,17 @@ done:
 	}
 
 	symbols_check_consistency(st);
+
+	/*
+	 * If symbols are garbage, retry with pre-computed symbol file.
+	 *
+	 * This usually happens when the executable has been stripped at
+	 * installation time but the BFD library still manages to find a
+	 * few symbols.
+	 */
+
+	if (!retried && !st->indirect && st->garbage)
+		goto use_pre_computed;
 }
 
 /**
