@@ -224,7 +224,7 @@ wq_timed_out(cqueue_t *unused_cq, void *arg)
 		return;
 	case WQ_EXCLUSIVE:
 		g_carp("weird status WQ_EXCLUSIVE on timeout invocation of %s()",
-			stacktrace_routine_name(func_to_pointer(we->cb), FALSE));
+			stacktrace_function_name(we->cb));
 		/* FALL THROUGH */
 	case WQ_REMOVE:
 		hash_list_remove(hl, we);
@@ -287,12 +287,10 @@ wq_remove(wq_event_t *we)
 	hl = htable_lookup(waitqueue, we->key);
 	if (NULL == hl) {
 		g_carp("attempt to remove event %s() on unknown key %p",
-			stacktrace_routine_name(func_to_pointer(we->cb), FALSE),
-			we->key);
+			stacktrace_function_name(we->cb), we->key);
 	} if (NULL == hash_list_remove(hl, we)) {
 		g_carp("attempt to remove unknown event %s() on %p",
-			stacktrace_routine_name(func_to_pointer(we->cb), FALSE),
-			we->key);
+			stacktrace_function_name(we->cb), we->key);
 	} else if (0 == hash_list_length(hl)) {
 		hash_list_free(&hl);
 		htable_remove(waitqueue, we->key);
@@ -347,7 +345,7 @@ wq_notify(hash_list_t *hl, void *data)
 
 		if (i++ >= count) {
 			/* Something is odd, let them know about the calling stack */
-			g_carp("stopping after processing %zu item%s (list now has %zu)",
+			g_carp("stopping after processing %zu item%s (list now has %u)",
 				count, 1 == count ? "" : "s", hash_list_length(hl));
 		}
 
@@ -362,7 +360,7 @@ wq_notify(hash_list_t *hl, void *data)
 		}
 
 		g_error("invalid status %d returned by %s()",
-			status, stacktrace_routine_name(func_to_pointer(we->cb), FALSE));
+			status, stacktrace_function_name(we->cb));
 
 	remove:
 		hash_list_iter_remove(iter);
@@ -447,8 +445,7 @@ wq_free_waiting(void *key, void *unused_data)
 	(void) unused_data;
 
 	g_warning("leaked waiting event %s() on %p",
-		stacktrace_routine_name(func_to_pointer(we->cb), FALSE),
-		we->key);
+		stacktrace_function_name(we->cb), we->key);
 
 	wq_event_free(we);
 	return TRUE;
