@@ -174,8 +174,6 @@ void search_gui_current_search_refresh(void);
 
 void search_gui_ref_record(record_t *);
 void search_gui_unref_record(record_t *);
-unsigned search_gui_hash_func(gconstpointer);
-int search_gui_hash_key_compare(gconstpointer, gconstpointer);
 const char *search_gui_get_route(const struct results_set *);
 const char *search_gui_get_filename_extension(const char *filename_utf8);
 void search_gui_set_sort_defaults(void);
@@ -261,7 +259,7 @@ typedef search_t *(*search_gui_synchronize_list_cb)(void *user_data);
 void search_gui_synchronize_search_list(search_gui_synchronize_list_cb,
 			void *user_data);
 
-void search_gui_start_massive_update(struct search *);
+bool search_gui_start_massive_update(struct search *);
 void search_gui_end_massive_update(struct search *);
 void search_gui_queue_bitzi_by_sha1(const record_t *);
 void search_gui_add_record(struct search *, record_t *, enum gui_color);
